//----------------------------------------------------------------------
// MainActivity.java
// Author: Bob Dondero
//----------------------------------------------------------------------

package edu.Princeton.penny;

import android.widget.EditText;
import android.app.Activity;
import android.os.Bundle;
import android.text.method.ScrollingMovementMethod;
import android.widget.TextView;
import android.text.TextWatcher;
import android.text.Editable;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import javax.net.ssl.HttpsURLConnection;
import java.util.Timer;
import java.util.TimerTask;

import org.json.JSONObject;
import org.json.JSONArray;

public class MainActivity extends Activity
{
   private EditText editText;
   private TextView textView;
   private AuthorSearchRunnable authorSearchRunnable = null;
   private Timer timer = null;

   //-------------------------------------------------------------------

   private class AuthorSearchRunnable implements Runnable
   {
      private String author;
      private boolean shouldStop = false;

      AuthorSearchRunnable(String author)
      {
         this.author = author;
      }

      public void setShouldStop()
      {
         shouldStop = true;
      }

      public void run()
      {
         String books;

         try
         {
            URL url = new URL("https://pennyjson.onrender.com" +
                              "/searchresults?author=" + author);
            HttpsURLConnection con =
               (HttpsURLConnection) url.openConnection();

            con.setRequestMethod("GET");
            con.setDoInput(true);
            con.connect();
            int responseCode = con.getResponseCode();
            if (responseCode != HttpsURLConnection.HTTP_OK)
               throw new IOException(
                  "HTTP error code: " + responseCode);

            // Read the response, a JSON document containing books,
            // from the server.
            BufferedReader in =
               new BufferedReader(
                  new InputStreamReader(con.getInputStream()));
            String jsonDoc = in.readLine();
            in.close();

            // Use the JSON response to build a String, and assign
            // it to books.
            JSONArray jsonArray = new JSONArray(jsonDoc);
            StringBuilder booksBuilder = new StringBuilder();
            for (int i = 0; i < jsonArray.length(); i++)
            {
               JSONObject jsonObject = jsonArray.getJSONObject(i);
               booksBuilder.append(jsonObject.getString("isbn"));
               booksBuilder.append(": ");
               booksBuilder.append(jsonObject.getString("author"));
               booksBuilder.append(": ");
               booksBuilder.append(jsonObject.getString("title"));
               booksBuilder.append("\n");
            }
            books = booksBuilder.toString();

            if (! shouldStop)
               runOnUiThread(() -> { textView.setText(books); });
         }

         catch (Exception e)
         {
            if (! shouldStop)
               runOnUiThread(() -> { textView.setText(e.toString()); });
         }
      }
   }

   //-------------------------------------------------------------------

   private class MyTimerTask extends TimerTask
   {
      public void run()
      {
         String author = editText.getText().toString();

         if (authorSearchRunnable != null)
            authorSearchRunnable.setShouldStop();

         authorSearchRunnable = new AuthorSearchRunnable(author);
         Thread authorSearchThread = new Thread(authorSearchRunnable);
         authorSearchThread.start();
      }
   }

   //-------------------------------------------------------------------

   private class MyTextWatcher implements TextWatcher
   {
      public void onTextChanged(CharSequence s, int start,
                                int before, int count)
      {
      }

      public void beforeTextChanged(CharSequence s, int start,
                                    int count, int after)
      {
      }

      public void afterTextChanged(Editable s)
      {
         if (timer != null)
            timer.cancel();
         timer = new Timer();
         timer.schedule(new MyTimerTask(), 500);
      }
   }

   //-------------------------------------------------------------------

   protected void onCreate(Bundle activityState)
   {
      super.onCreate(activityState);

      // Set the content view of the window to activity_main,
      // as defined in activity_main.xml.
      setContentView(R.layout.activity_main);

      // Find the EditText object.
      editText = findViewById(R.id.editText);

      // Instantiate a TextWatcher object, and install it as a
      // "text changed" listener on the EditText object. Subsequently,
      // immediately after the user changes the text within the
      // EditText object, the afterTextChanged() method will be called.
      editText.addTextChangedListener(new MyTextWatcher());

      // Find the TextView object.
      textView = findViewById(R.id.textView);

      // Allow the TextView object to scroll, just in case there are
      // many books.
      textView.setMovementMethod(new ScrollingMovementMethod());
   }
}
